package com.zhongzheng.modules.activity.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.zhongzheng.modules.activity.bo.ActivityConfigurationQueryBo;
import com.zhongzheng.modules.activity.domain.ActivityConfiguration;
import com.zhongzheng.modules.activity.vo.ActivityConfigurationVo;
import com.zhongzheng.modules.course.bo.CourseQueryBo;
import com.zhongzheng.modules.course.vo.CourseConfigurationVo;
import org.apache.ibatis.annotations.Param;

import java.util.Collection;
import java.util.List;

/**
 * 活动配置Mapper接口
 *
 * @author change
 * @date 2021-05-17
 */
public interface ActivityConfigurationMapper extends BaseMapper<ActivityConfiguration> {

    /**
     * 查询配置分页
     *
     * @param bo 配置信息
     * @return 配置信息集合信息
     */
    Collection<ActivityConfigurationVo> selectConfiguraList(ActivityConfigurationQueryBo bo);

    /**
     * 详细配置
     *
     * @param id 详细配置信息
     * @return 详细配置
     */
    ActivityConfigurationVo selectConfigures(Long id);


    /**
     * 查询用户能做的活动
     *
     * @param bo 详细配置信息
     * @return Collection<CourseConfigurationVo> 详细配置
     */
    Collection<ActivityConfigurationVo> selectConfiguraIssueList(ActivityConfigurationQueryBo bo);


    /**
     * 查询课程能做的活动
     *
     * @param bo 详细配置信息
     * @return Collection<CourseConfigurationVo> 详细配置
     */
    List<ActivityConfigurationVo> selectlistConfiguration(CourseQueryBo bo);

    /**
     *
     * @author change
     * @date 2021/7/1 16:14
     * @param goodsId
     * @param activity
     * @return int
     */
    int selectCountActivity(@Param("goodsId") Long goodsId, @Param("activityId") Long activity);

    long selectConfiguraCount(ActivityConfigurationQueryBo bo);
}
